/* $Id: osf1.h,v 1.3 1998/11/05 02:06:10 marcus Exp $
******************************************************************************

   KGI - Digital Unix/OSF1 specific parts

   Copyright (C) 1998 Marcus Sundberg	[marcus@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#ifndef _KGI_SYS_OSF1_H
#define _KGI_SYS_OSF1_H

#ifndef __KERNEL__
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#endif


/*
***************************************************************************
 Random Linux specific junk with random values
***************************************************************************
*/

#define MAX_NR_CONSOLES		16
struct pt_regs {
	int dummy;
};
#define	MMIO_RD		0
#define	MMIO_WR		0
#define	MMIO_RDWR	0
#define	MMIO_SHARED	0

#define PCI_DEVFN(slot,fn)	LINUX_JUNK

#define	IS_CRITICAL     int unused_crap
#define	ENTER_CRITICAL	unused_crap = 1
#define	LEAVE_CRITICAL	unused_crap = 0

#define PCI_STATUS_UDF			0
#define PCI_DEVICE_ID_MATROX_MIL	0

#define PCI_VENDOR_ID_CIRRUS		0
#define PCI_VENDOR_ID_MATROX		0

#define PCI_COMMAND_MEMORY		0
#define PCI_COMMAND_IO			0
#define PCI_COMMAND			0
#define PCI_CLASS_REVISION		0
#define PCI_DEVICE_ID_MATROX_MYS	0
#define PCI_BASE_ADDRESS_0		0
#define PCI_BASE_ADDRESS_1		0
#define PCI_INTERRUPT_LINE		0
#define PCIBIOS_SUCCESSFUL		0

#define SA_SHIRQ			0

/* Undefined functions */

static inline void *
phys_to_virt(unsigned long address)
{
	return NULL;
}

static inline int
suser(void)
{
	return 1;
}

static inline int
pcibios_present (void)
{
	return 0;
}

static inline int
pcibios_find_device (unsigned short vendor, unsigned short dev_id,
		     unsigned short index, unsigned char *bus,
		     unsigned char *dev_fn)
{
	return 0;
}

static inline int
pcibios_read_config_byte (unsigned char bus, unsigned char dev_fn,
			  unsigned char where, unsigned char *val)

{
	return 0;
}

static inline int
pcibios_read_config_word (unsigned char bus, unsigned char dev_fn,
			  unsigned char where, unsigned short *val)
{
	return 0;
}

static inline int
pcibios_read_config_dword(unsigned char bus, unsigned char dev_fn,
			  unsigned char where, unsigned int *val)
{
	return 0;
}

static inline int
pcibios_write_config_word(unsigned char bus, unsigned char dev_fn,
			       unsigned char where, unsigned short val)
{
	return 0;
}

static inline int
pcibios_write_config_dword(unsigned char bus, unsigned char dev_fn,
			   unsigned char where, unsigned int val)
{
	return 0;
}


static inline int
request_irq(unsigned int irq, void (*handler)(int, void *, struct pt_regs *),
	    unsigned long flags, const char *device, void *dev_id)
{
	return 0;
}

static inline void
free_irq(unsigned int irq, void *dev_id)
{
}


/*
***************************************************************************
 Misc stuff
***************************************************************************
*/

typedef kgi_addr mem_vaddr;

#define kgi_malloc_small(size)	malloc(size)
#define	kgi_free_small(addr)	free(addr)

#define	kgi_malloc_huge(size)	malloc(size)
#define	kgi_free_huge(addr)	free(addr)

/*
***************************************************************************
 Intel style I/O functions
***************************************************************************
*/

typedef kgiu16 io_addr; 	/* the physical address type	*/
typedef kgiu16 io_vaddr; 	/* the virtual address type	*/

struct io_region 
{
	io_vaddr baseptr;	/* virtual base address		*/
	io_addr base;		/* physical base address	*/
	io_addr size;		/* size of region		*/
	io_addr decode;		/* decoded phys address lines	*/
	char *name;		/* name of the region		*/
};

#define	IO_NULL		((io_vaddr) 0)

static inline int io_check_region(struct io_region *r)
{
	return 0;
}

static inline io_vaddr io_claim_region(struct io_region *r)
{
	return NULL;
}

static inline io_vaddr io_free_region(struct io_region *r)
{
	return NULL;
}

static inline io_vaddr io_alloc_region(struct io_region *r)
{
	return IO_NULL;
}

static inline kgiu8 io_in8(io_vaddr addr) 
{
	return 0;
}

static inline kgiu16 io_in16(io_vaddr addr)
{
	return 0;
}

static inline kgiu32 io_in32(io_vaddr addr)
{
	return 0;
}

static inline void io_ins8(io_vaddr addr, void *buf, kgi_uint cnt)
{
}

static inline void io_ins16(io_vaddr addr, void *buf, kgi_uint cnt)
{
}

static inline void io_ins32(io_vaddr addr, void *buf, kgi_uint cnt)
{
}

static inline void io_out8(kgiu8 val, io_vaddr addr)
{
}

static inline void io_out16(kgiu16 val, io_vaddr addr)
{
}

static inline void io_out32(kgiu32 val, io_vaddr addr)
{
}

static inline void io_outs8(io_vaddr addr, void *buf, kgi_uint cnt)
{
}

static inline void io_outs16(io_vaddr addr, void *buf, kgi_uint cnt)
{
}

static inline void io_outs32(io_vaddr addr, void *buf, kgi_uint cnt)
{
}


/*
***************************************************************************
 Memory I/O functions
***************************************************************************
*/

typedef kgi_addr mem_addr; 	/* the physical address type	*/

struct mem_region 
{
	mem_vaddr baseptr;	/* virtual base address		*/
	mem_addr base;		/* physical base address	*/
	mem_addr size;		/* size of region		*/
	mem_addr decode;	/* decoded phys address lines	*/
	char *name;		/* name of the region		*/
};

#define	MEM_NULL	((mem_vaddr) 0)

static inline kgi_sint mem_check_region(struct mem_region *r)
{
	return 0;
}

static inline mem_vaddr mem_claim_region(struct mem_region *r)
{
	return NULL;
}

static inline mem_vaddr mem_free_region(struct mem_region *r)
{
	return NULL;
}

static inline mem_vaddr mem_alloc_region(struct mem_region *r)
{
	return MEM_NULL;
}

static inline kgiu8 mem_in8(kgi_addr addr)
{
	return 	0;
}

static inline kgiu16 mem_in16(kgi_addr addr)
{
	return 0;
}

static inline kgiu32 mem_in32(kgi_addr addr)
{
	return 0;
}

static inline void mem_out8(kgiu8 val, kgi_addr addr)
{
}

static inline void mem_out16(kgiu16 val, kgi_addr addr)
{
}

static inline void mem_out32(kgiu32 val, kgi_addr addr)
{
}

#define	MEM_INS(x) \
static inline void mem_ins##x(mem_vaddr addr, kgiu##x *buf, int cnt)	\
{								\
	while (cnt--) {						\
								\
		*(buf++) = mem_in##x(addr);			\
	}							\
}

#define	MEM_OUTS(x) \
static inline void mem_outs##x(mem_vaddr addr, kgiu##x *buf, int cnt)	\
{								\
	while (cnt--) {						\
								\
		mem_out##x(*(buf++), addr);			\
	}							\
}

MEM_INS(8)
MEM_INS(16)
MEM_INS(32)

MEM_OUTS(8)
MEM_OUTS(16)
MEM_OUTS(32)
#undef	MEM_OUTS
#undef	MEM_INS


/*
***************************************************************************
 PCI I/O functions
***************************************************************************
*/

typedef kgiu8 *pcicfg_vaddr;

#define	PCICFG_NULL	((pcicfg_vaddr) -1)
#define	PCICFG_VADDR(bus, slot, fn)	\
	((pcicfg_vaddr)((((bus) & 0xFF) << 24) | (PCI_DEVFN(slot,fn) << 16)))

#define	PCIARGS	(addr >> 24) & 0xFF, (addr >> 16) & 0xFF, addr & 0xFF

static inline kgiu8 pcicfg_in8(pcicfg_vaddr addr)
{
	return -1;
}

static inline kgiu16 pcicfg_in16(pcicfg_vaddr addr)
{
	return -1;
}

static inline kgiu32 pcicfg_in32(pcicfg_vaddr addr)
{
	return -1;
}

static inline void pcicfg_out8(kgiu8 val, pcicfg_vaddr addr)
{
}

static inline void pcicfg_out16(kgiu16 val, pcicfg_vaddr addr)
{
}

static inline void pcicfg_out32(kgiu32 val, pcicfg_vaddr addr)
{
}


#endif /* _KGI_SYS_OSF1_H */



