/*******************************************************************************
 *	ICS 5342 PLL driver
 *******************************************************************************
 *
   Copyright (C) 1997-1998		Emmanuel Marty
   Copyright (C) 1997-1998		The GGI project
 *
   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 *******************************************************************************
 *
 *	This code was derived from the following sources of information :
 *
 *	ICS 5342 datasheet (www.icst.com)
 *
 */

#include <kgi/maintainers.h>
#define MAINTAINER	Emmanuel_Marty
#define DRIVER_NAME	"ICS 5342 PLL driver"
#define DRIVER_REV	"$Revision: 1.6 $"
#define DEBUG_LEVEL	0

#include <kgi/module.h>
#include <ICS/5342.h>
#include "pll.h"

#define	INVALID_COMMAND	((kgiu32) -1)

static struct kgi_clock clock =
{
	"Integrated Circuit Systems",		/* manufacturer		*/
	"ICS 5342",				/* model		*/
	GM_ALL,					/* supported modes 	*/
	0,					/* special capabilities	*/
	{ 0x7FFF, 0x7FFF },			/* maximum resolution	*/
	NULL,					/* Private data hook	*/
	CLK_TYPE_PROG,				/* clock type		*/
	{{
		25000000, 270000000,		/* clock range		*/
	}}
};

static const struct pll_limits dclk_pll = 
{
	14318180,			/* fref				*/
	{ 1, 1 },			/* factor		 	*/
	{ 60000000, 270000000 },	/* fvco range			*/
	{ 0, 3 },			/* post-VCO divider range	*/
	{ 3, 129 },			/* dividend range		*/
	{ 3, 33 }			/* divisor range		*/
};

/*
**	An ICS 5342 command word is as follows :
**
**	[0:6]	s->mul - 2 (M)
**	[8:12]	s->div - 2 (N1)
**	[13:14]	s->p       (N2)
*/

static kgiu32 pll_command (struct pll_state *s, struct kgi_mode *mode) {
	ASSERT ( (s->mul - 2) < 128);
	ASSERT ( (s->div - 2) < 32);
	ASSERT ( s->p < 4);

	return	(s->mul - 2) | ((s->div - 2) << 8) | (s->p << 13);
}

static void pll_set (struct kgi_display *dpy, kgi_uint cmd) {
	/*	select rate from clock 0 	*/

	CLK_out8 (dpy, 0x20, PLL_CONTROL);

	/*	program clock 2 		*/

	CLK_out8 (dpy, cmd & 0xFF, PLL_CLK0_F2);
	CLK_out8 (dpy, cmd >> 8, PLL_CLK0_F2);

	/*	select rate from clock 2 	*/

	CLK_out8 (dpy, 0x22, PLL_CONTROL);
}

#include "pll.inc"
