/* $Id: s3idac.c,v 1.5 1998/11/23 20:37:12 marcus Exp $
******************************************************************************

   Trio/ViRGE integrated PLL driver

   Copyright (C) 1997 Steffen Seeger	[seeger@ggi-project.org]
   Copyright (C) 1997 Jon M. Taylor	[taylorj@csus.edu]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************

   This code was derived from the following source of information:
   [1] SDAC ramdac/clock synthesizer data book

******************************************************************************
*/

#include <kgi/maintainers.h>
#define	MAINTAINER	Jon_Taylor
#define	DRIVER_NAME	"S3 Trio/ViRGE integrated PLL clock driver"
#define	DRIVER_REV	"$Revision: 1.5 $"

#define	DEBUG_LEVEL	0

#include <kgi/module.h>

#include <S3/idac.h>
#include "pll.h"

static const struct pll_limits dclk_pll = 
{
	14318180,			/* Fref				*/
	{ 1, 1 },			/* Factor		 	*/
	{ 135000000, 270000000 },	/* FVCO range			*/
	{ 0, 3 },			/* Post-VCO divider range	*/
	{ 3, 129 },			/* Dividend range		*/
	{ 3, 33 }			/* Divisor range		*/
};

static struct kgi_clock clock =
{
	"S3 Incorporated",		/* Manufacturer		*/
	"Trio/ViRGE integrated DAC",	/* Model		*/
	GM_ALL,				/* Supported modes 	*/
	0,				/* Special capabilities	*/
	{ 0x7FFF, 0x7FFF },		/* Maximum resolution	*/
	NULL,				/* Private data hook	*/
	CLK_TYPE_PROG,			/* Clock type		*/
	{{
		25000000, 110000000,	/* Clock range		*/
	}}
};

/* An SDAC command word is as follows:
**
**	[0:6]	s->mul - 2	
**	[8:12]	s->div - 2
**	[13:14]	s->p
*/

#define	INVALID_COMMAND	((kgi_uint) -1)

static kgi_uint pll_command(struct pll_state *s, struct kgi_mode *mode)
{
	ASSERT(s->mul - 2 < 128);
	ASSERT(s->div - 2 < 32);
	ASSERT(s->p < 4);

	return	(s->mul - 2) | ((s->div - 2) << 8) | (s->p << 13);
}

static void pll_set(struct kgi_display *dpy, kgi_uint cmd)
{
	/* Set clock select lines */
	CLK_out8(dpy, CLK0_PLL, 0);

	/* Set DCLK PLL parameters */
	CLK_out8(dpy, (cmd >> 8) & (CLK3_DCLK_N_MASK | CLK3_DCLK_R_MASK), 3);
	CLK_out8(dpy, cmd & CLK4_DCLK_M_MASK, 4);

	/* Load values */
#ifdef	DELAYED_LOAD
	CLK_out8(dpy, CLK6_LOAD_DLCK, 6);
#else
	CLK_out8(dpy, CLK6_LOAD_CLKS, 6);
	CLK_out8(dpy, CLK6_LOAD_DCLK, 6);
#endif
}

#include "pll.inc"
